"""
Since the precision code is so unusual, I decided to create a class
to parse it.
"""

#from .GimpImageHierarchy import GimpImageHierarchy

class Precision:
	"""
	Since the precision code is so unusual, I decided to create a class
	to parse it.
	"""
	def __init__(self):
		self.bits = 8
		self.gamma = True
		self.numberFormat = int

	def decode_(self, gimpVersion, io):
		"""
		decode the precision code from the file
		"""
		if gimpVersion < 4:
			self.bits = 8
			self.gamma = True
			self.numberFormat = int
		else:
			code = io.u32
			if gimpVersion == 4:
				self.gamma = (True, True, False, False, False)[code]
				self.bits = (8, 16, 32, 16, 32)[code]
				self.numberFormat = (int, int, int, float, float)[code]
			elif gimpVersion in (5, 6):
				self.gamma = (code % 100 != 0)
				code = int(code / 100)
				self.bits = (8, 16, 32, 16, 32)[code]
				self.numberFormat = (int, int, int, float, float)[code]
			else: # gimpVersion 7 or above
				self.gamma = (code % 100 != 0)
				code = int(code / 100)
				self.bits = (8, 16, 32, 16, 32, 64)[code]
				self.numberFormat = (int, int, int, float, float, float)[code]

	def encode_(self, gimpVersion, io):
		"""
		encode this to the file

		NOTE: will not mess with development versions 5 or 6
		"""
		if gimpVersion < 4:
			if self.bits != 8 or not (self.gamma) or self.numberFormat != int:
				raise Exception('Illegal precision (' + str(self) + ') for gimp version ' +
				str(gimpVersion))
		else:
			if gimpVersion == 4:
				if self.bits == 64:
					raise Exception('Illegal precision (' + str(self) + ') for gimp version ' +
					str(gimpVersion))
				if self.numberFormat == int:
					code = (8, 16, 32).index(self.bits)
				else:
					code = (16, 32).index(self.bits) + 2
				code = code * 100
				if self.gamma:
					code += 50
			elif gimpVersion in (5, 6):
				raise NotImplementedError('Cannot save to gimp developer version ' +
				str(gimpVersion))
			else: # version 7 or above
				if self.numberFormat == int:
					code = (8, 16, 32).index(self.bits)
				else:
					code = (16, 32, 64).index(self.bits) + 2
				code = code * 100
				if self.gamma:
					code += 50
			io.u32 = code

	def requiredGimpVersion(self):
		"""
		return the lowest gimp version that supports this precision
		"""
		if self.bits == 8 and self.gamma and self.numberFormat == int:
			return 0
		if self.bits == 64:
			return 7
		return 4

	def __repr__(self):
		ret = []
		ret.append(str(self.bits) + "-bit")
		ret.append('gamma' if self.gamma else 'linear')
		ret.append('integer' if self.numberFormat is int else float)
		return ' '.join(ret)



"""
Gets packed pixels from a gimp image

NOTE: This was originally designed to be a hierarchy, like
	an image pyramid, through in practice they only use the
	top level of the pyramid (64x64) and ignore the rest.
"""

from .GimpIOBase import IO, GimpIOBase
from .GimpImageLevel import GimpImageLevel


class GimpImageHierarchy(GimpIOBase):
	"""
	Gets packed pixels from a gimp image

	NOTE: This was originally designed to be a hierarchy, like
		an image pyramid, through in practice they only use the
		top level of the pyramid (64x64) and ignore the rest.
	"""
	def __init__(self, parent, image=None):
		GimpIOBase.__init__(self, parent)
		self.width = 0
		self.height = 0
		self.bpp = 0 # Number of bytes per pixel given
		self._levelPtrs = []
		self._levels = None
		self._data = None
		if image is not None: # NOTE: can override earlier parameters
			self.image = image

	def decode_(self, data, index=0):
		"""
		decode a byte buffer

		:param data: data buffer to decode
		:param index: index within the buffer to start at
		"""
		if not data:
			raise Exception('No data!')
		io = IO(data, index)
		#print 'Decoding channel at',index
		self.width = io.u32
		self.height = io.u32
		self.bpp = io.u32
		if self.bpp < 1 or self.bpp > 4:
			msg = """'Unexpected bytes-per-pixel for image data (""" + str(self.bpp) + """).
				Probably means file corruption."""
			raise Exception(msg)
		while True:
			ptr = self._pointerDecode_(io)
			if ptr == 0:
				break
			self._levelPtrs.append(ptr)
		if self._levelPtrs: # remove "dummy" level pointers
			self._levelPtrs = [self._levelPtrs[0]]
		self._data = data
		return io.index

	def encode_(self):
		"""
		encode this object to a byte buffer
		"""
		dataIO = IO()
		io = IO()
		io.u32 = self.width
		io.u32 = self.height
		io.u32 = self.bpp
		dataIndex = io.index + GimpIOBase._POINTER_SIZE_ * (len(self.levels) + 1)
		for level in self.levels:
			io.addBytes(self._pointerEncode_(dataIndex + dataIO.index)) # TODO: This may be incorrect
			dataIO.addBytes(level.encode_())
		io.addBytes(self._pointerEncode_(0))
		io.addBytes(dataIO.data)
		return io.data

	@property
	def levels(self):
		"""
		Get the levels within this hierarchy

		Presently hierarchy is not really used by gimp,
		so this returns an array of one item
		"""
		if self._levels is None:
			for ptr in self._levelPtrs:
				l = GimpImageLevel(self)
				l.decode_(self._data, ptr)
				self._levels = [l]
		return self._levels

	@property
	def image(self):
		"""
		get a final, compiled image
		"""
		if not self.levels:
			return None
		return self.levels[0].image

	@image.setter
	def image(self, image):
		"""
		set the image
		"""
		self.width = image.width
		self.height = image.height
		if image.mode not in ['L', 'LA', 'RGB', 'RGBA']:
			raise NotImplementedError('Unsupported PIL image type')
		self.bpp = len(image.mode)
		self._levelPtrs = None
		#self._levels = [GimpImageLevel(self, image)]

	def __repr__(self, indent=''):
		"""
		Get a textual representation of this object
		"""
		ret = []
		ret.append('Size: ' + str(self.width) + ' x ' + str(self.height))
		ret.append('Bytes Per Pixel: ' + str(self.bpp))
		return indent + (('\n' + indent).join(ret))
